--[[
## Title:	Utility Script
## Author:	ThundR75
--]]

function getIsBoolean(arg)
-- This function simply returns whether or not the given argument is a boolean value
	--[arg]: Argument to check

-- Returns true if argument is a boolean value

	if type(arg) == "boolean" then
		return true
	end
end

function getIsString(arg)
-- This function simply returns whether or not the given argument is a string value
	--[arg]: Argument to check

-- Returns true if argument is a string value

	if type(arg) == "string" then
		return true
	end
end

function getIsNumber(arg)
-- This function simply returns whether or not the given argument is a number value
	--[arg]: Argument to check

-- Returns true if argument is a number value

	if tonumber(arg) then
		return true
	end
end

function getIsTable(arg)
-- This function simply returns whether or not the given argument is a table value
	--[arg]: Argument to check

-- Returns true if argument is a table value

	if type(arg) == "table" then
		return true
	end
end

function raiseError(text, showCallstack, ...)
-- This function halts operation and displays an error message (optionally showing the current callstack)
	-- [text]:			Text to display on error
	-- [showCallstack]:	Shows the current callstack when true
	-- [...]:			Variables to pass to the built-in string format method

	if (text and not getIsString(text)) or (showCallstack and not getIsBoolean(showCallStack)) then
		raiseError()
		return
	end

	if not text and showCallstack == nil then
		showCallstack = true
	end

	text = Utils.getNoNil(text, globalErrorText)
	if getIsString(text) and g_i18n:hasText(text) then
		text = g_i18n:getText(text)
	else
		text = "An internal error has occurred."
	end

	local title = ""

	if getIsTable(globalCaller) and globalCaller.title then
		title = title..tostring(globalCaller.title)
		if globalCaller.version then
			title = title.." v"..tostring(globalCaller.version)
		end
		title = title..": "
	end

	if showCallstack then
		printCallstack()
	end

	error(">>>> "..string.format("%s%s", title, string.format(text, ...)), 0)
end

function xAssert(arg, text, showCallstack, ...)
-- Similar to raiseError, this is a wrapper function for the assert method that evaluates a line of code and raises an error on nil or false
	-- [arg]:			Code to check
	-- [text]:			Text to display on error
	-- [showCallstack]:	Displays the current callstack if true
	-- [...]:			Variables to pass to the built-in string.format method

-- Returns the result of the code passed or error on false or nil

	if not arg then
		raiseError(text, showCallstack, ...)
		return
	end
	return arg
end

function debugMsg(lvl, text, ...)
-- This function displays a message without raising an error
	-- [lvl]:	If mod debugLvl value is higher than this number, the message will be shown
	-- [text]:	Text to display
	-- [...]:	Variables to pass to the built-in string.format method

	if not getIsNumber(lvl) or not getIsString(text) then
		raiseError()
		return
	end

	if g_i18n:hasText(text) then
		text = g_i18n:getText(text)
	end

	local title = ""
	local debugLvl = 0

	if getIsTable(globalCaller) then
		if getIsNumber(globalCaller.debugLvl) then
			debugLvl = globalCaller.debugLvl
		end
		if globalCaller.title then
			title = title..tostring(globalCaller.title)
			if globalCaller.version then
				title = title.." v"..tostring(globalCaller.version)
			end
			title = title..": "
		end
	end

	if lvl <= debugLvl then
		print(">>>> "..string.format("%s%s", title, string.format(text, ...)))
	end
end

function getText(text, ...)
-- Wrapper function for g_i18n:getText with string formatting features built in
	-- [text]: Name of text to retrieve
	-- [...]:  Variables to pass to the built-in string.format feature

-- Returns the formatted g_i18n text

	return string.format(g_i18n:getText(text), ...)
end

function strToNum(str)
-- Converts a string value to a numeric value, removing any non-numeric characters
	-- [str]: String to convert

-- Returns the converted numeric value

	if str == nil or (not getIsNumber(str) and not getIsString(str)) then
		raiseError()
		return
	end

	str = tostring(str)
	str = string.gsub(str, ",", "")
	str = string.match(str, "%-?%d+%.?%d*[eE]?%-?%d*")

	return tonumber(str)
end

function numToStr(num, arg, roundType, argIsPrecision, showCommas)
-- This function converts and formats a numeric value
	-- [num]:            Numeric value to convert
	-- [arg]:            This value can be precision or string length depending on the value of [argIsPrecision]
	-- [roundType]:      This value can be "round", "floor", or "ceil" and determines how a number is truncated
	-- [argIsPrecision]: If true, the value of [arg] is a precision value. Otherwise, [arg] is overall string length
	-- [showCommas]:     If true, will separate large numbers with commas

-- Returns the formatted number as a string

	if arg == nil then arg = 0 end

	if not getIsNumber(num) or not getIsNumber(arg)
	or (argIsPrecision and not getIsBoolean(argIsPrecision))
	or (showCommas and not getIsBoolean(showCommas))
	then
		raiseError()
		return
	end

	local precision = arg
	if not argIsPrecision then
		precision = arg - string.len(math.floor(num))
		precision = math.max(0, precision)
	end

	if roundType == "round" then
		num = math.floor((num * (10 ^ precision)) + 0.5) / (10 ^ precision)
	elseif roundType == "floor" then
		num = math.floor(num * (10 ^ precision)) / (10 ^ precision)
	else
		num = math.ceil(num * (10 ^ precision)) / (10 ^ precision)
	end

	local num = string.format("%."..tostring(precision).."f", num)

	if showCommas then
		local v = 0
		while true do
			num, v = string.gsub(num, "^(%-?%d+)(%d%d%d)", "%1,%2")
			if v == 0 then break end
		end
	end

	return tostring(num)
end

function getTableLength(tbl)
-- This is a simple function that returns the length (number of entries) in the given table
-- Used for tables that have non-consecutive keys
	-- [tbl] Table to check

-- Returns the length of the given table

	if not getIsTable(tbl) then
		raiseError()
	end

	local idx = 0
	for k, v in pairs(tbl) do
		idx = idx + 1
	end

	return idx
end

function splitString(pattern, text)
-- This function splits a list of values into a table of values
	-- [pattern]: The string pattern used to separate values
	-- [text]:    The string of values to split

-- Returns a table containing the list of entries specified by [text]

	if not getIsString(pattern) or text == nil or (not getIsNumber(text) and not getIsString(text)) then
		raiseError()
		return
	end

	text = tostring(text)
	local retTbl = Utils.splitString(pattern, text)

	for i = 1, getTableLength(retTbl) do
		retTbl[i] = Utils.getNoNil(tonumber(retTbl[i]), retTbl[i])
	end

	return retTbl
end

function sortTable(tbl, key)
-- This function sorts a table by the specified key
	-- [tbl]: Table to sort
	-- [key]: Key of table used as the basis for sorting

-- Returns a table sorted by the specified [key]

	if not getIsTable(tbl) or key == nil or (not getIsNumber(key) and not getIsString(key)) then
		raiseError()
		return
	end

	local tmpTbl = {}
	for k, v in pairs(tbl) do
		local itemToAdd = v[key]
		if itemToAdd then
			local added = false
			for _, value in pairs(tmpTbl) do
				if itemToAdd == value then
					added = true
				end
			end
			if not added then
				table.insert(tmpTbl, itemToAdd)
			end
		end
	end

	table.sort(tmpTbl)

	local tblSorted = {}
	for _, value in ipairs(tmpTbl) do
		for k, v in pairs(tbl) do
			if v[key] == value then
				table.insert(tblSorted, v)
			end
		end
	end

	return tblSorted
end

function indexTableByKey(tbl, key)
-- This function creates a table indexed by the values of the given key
	-- [tbl]: Source table used for indexing
	-- [key]: Table key used for indexing

-- Returns a table indexed by the specified [key]

	if not getIsTable(tbl) or key == nil or (not getIsNumber(key) and not getIsString(key)) then
		raiseError()
		return
	end

	local maxVal = 0

	local tblTemp = {}
	for k, v in pairs(tbl) do
		local idx = tonumber(v[key])
		if idx then
			idx = idx * getTableLength(tbl)
			while tblTemp[idx] do
				idx = idx + 1
			end
			tblTemp[idx] = v
			maxVal = math.max(maxVal, idx)
		end
	end

	local idx = 1
	local tblIndexed = {}
	for i = 1, maxVal do
		if tblTemp[i] then
			tblIndexed[idx] = tblTemp[i]
			tblIndexed[idx][key] = idx
			idx = idx + 1
		end
	end

	return tblIndexed
end
