--[[
## Title:	UnitConvert Settings Menu
## Author:	ThundR75
--]]

UCMenu = {}

local ucMenu_mt = Class(UCMenu, ScreenElement)
local specialFunctionKey = {Input.KEY_lshift}

-- [MAIN] ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:new(target, custom_mt)
    if custom_mt == nil then
        custom_mt = ucMenu_mt
    end
    local self = ScreenElement:new(target, custom_mt)
    return self
end

function UCMenu:onCreate(gui)
	-- Set constants
	self.DOUBLECLICK_TIME = 0.5

	-- Set menu title
	self.menuTitle:setText(getText("UC_menu_title", ucMod.version))
	self.menuTitleShadow:setText(self.menuTitle.text)

	-- Create list table
	self.listElements = {}
	table.insert(self.listElements, "fillTypes")
	table.insert(self.listElements, "units")
	table.insert(self.listElements, "global")
	table.insert(self.listElements, "settings")
	table.insert(self.listElements, "groups")
	for k, v in pairs(self.listElements) do
		self["LIST_"..string.upper(v)] = k
	end

	-- Create fillType sorting table
	self.fillTypeSortTypes = {}
	table.insert(self.fillTypeSortTypes, "index")
	table.insert(self.fillTypeSortTypes, "name")
	table.insert(self.fillTypeSortTypes, "group")
	for k, v in pairs(self.fillTypeSortTypes) do
		self["FILLTYPE_SORT_"..string.upper(v)] = k
	end
	self.fillTypeSortType = self.FILLTYPE_SORT_INDEX
end

function UCMenu:onOpen(element)
	UCMenu:superClass().onOpen(self)

	self:initializeSettingsList()

	for k, v in ipairs(self.listElements) do
		if k ~= self.LIST_SETTINGS then
			self:initializeList(self[v.."List"], 1, true)
		end
	end

	local focusedElement = FocusManager:getFocusedElement()
	if focusedElement then
		FocusManager:unsetFocus(focusedElement)
	end
	FocusManager:setFocus(self.fillTypesList)

	if DebugTools then
		--DebugTools:searchTable(self, 0, nil, true)
	end
end

function UCMenu:reloadMenu()
-- This function reloads the UnitConvert menu, refreshing its values

	self:initializeSettingsList()

	for k, v in ipairs(self.listElements) do
		if k ~= self.LIST_SETTINGS then
			self:initializeList(self[v.."List"])
		end
	end
end

function UCMenu:update(dt)
	-- Hit "cancel" button on ESC press
	if InputBinding.hasEvent(InputBinding.MENU_CANCEL) then
		local cancelButton = FocusManager:getElementById("cancelButton")
		cancelButton:raiseCallback("onClickCallback")
	end

	-- Update lists
	for k, v in ipairs(self.listElements) do
		self:updateList(self[v.."List"], dt)
	end

	-- Update toolTips
	local isToolTipVisible = false
	local focusedElement = FocusManager:getFocusedElement()
	if focusedElement then
		local toolTip = focusedElement.toolTip
		if focusedElement.listItems then -- If a list is selected
			local selectedElement = focusedElement:getSelectedElement()
			if selectedElement then
				toolTip = selectedElement.toolTip
			end
		end
		if toolTip then
			local textElement = self.toolTipBox.textElement
			textElement:setText(toolTip)
			isToolTipVisible = true
		end
	end

	if isToolTipVisible ~= self.toolTipBox:getIsVisible() then
		self.toolTipBox:setVisible(isToolTipVisible)
	end
end

function UCMenu:saveChanges(yes)
	if yes then
		ucMod:saveConfig()
		g_gui:showGui("")
	end
end

function UCMenu:cancelChanges(yes)
	if yes then
		ucMod:loadConfig()
		g_gui:showGui("")
	end
end

function UCMenu:loadDefaults(yes)
	if yes then
		ucMod:loadConfig(true)
		self:reloadMenu()
	end
end

function UCMenu:onClose(element)
	UCMenu:superClass().onClose(self)
end

-- [BUTTONS] ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:hasDoubleClick(element)
-- This function simulates a double click event (because buttons don't seem to have one by default, or I can't find it)
-- Returns true if button (or any click-able element) was double clicked

	if not element.onClickCallback then
		return
	end

	local clickTime = getTimeSec()
	if element.lastClickTime and math.abs(clickTime - element.lastClickTime) <= self.DOUBLECLICK_TIME then
		element.lastClickTime = 0
		return true
	else
		element.lastClickTime = clickTime
		return false
	end
end

function UCMenu:onCreateDummyButton(element)
	element.handleFocus = false
end

function UCMenu:onCreatePrevButton(element)
	element.valueDirection = -1
end

function UCMenu:onCreateNextButton(element)
	element.valueDirection = 1
end

function UCMenu:onCreateListItemPrevButton(element)
	element.handleFocus = false
	element.valueDirection = -1
end

function UCMenu:onCreateListItemNextButton(element)
	element.handleFocus = false
	element.valueDirection = 1
end

function UCMenu:onClickSave(element)
	g_gui:showYesNoDialog({text = getText("UC_dialog_saveText"), title = "", callback = self.saveChanges, target = self})
end

function UCMenu:onClickCancel(element)
	g_gui:showYesNoDialog({text = getText("UC_dialog_cancelText"), title = "", callback = self.cancelChanges, target = self})
end

function UCMenu:onClickDefaults(element)
	g_gui:showYesNoDialog({text = getText("UC_dialog_defaultsText"), title = "", callback = self.loadDefaults, target = self})
end

-- [TOOLTIP] ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onCreateToolTipBox(element)
	element.textElement = element.elements[2]
end

-- [LIST] ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onCreateListItemValues(element)
	element.valueIndex = 1
	element.textElement = element.elements[3]
end

function UCMenu:initializeList(list, selectedItem, doNotUpdateSlider)
-- This function initializes and refreshes the UnitConvert menu lists
	-- [list]:              List to initialize
	-- [selectedItem]:      Item to select on completion
	-- [doNotUpdateSlider]: If true, does not move the slider to its position before updating

	if not getIsTable(list)
	or (selectedItem and not getIsNumber(selectedItem))
	or (doNotUpdateSlider and not getIsBoolean(doNotUpdateSlider))
	then
		raiseError()
		return
	end

	selectedItem = Utils.getNoNil(selectedItem, Utils.getNoNil(list.selectedRow, 1))

	local slider = list.parent
	local sliderValue = slider.currentValue

	if not self:getIsListItemVisible(list, selectedItem) or doNotUpdateSlider then
		sliderValue = nil
	end

	list:deleteListItems()
	local sourceTable = nil
	local template = nil

	if list == self.fillTypesList then
		list.fillTypeMapping = {}
		list.highlight = self.fillTypesBoxHighlight

		sourceTable = ucMod.fillTypesByIndex
		if self.fillTypeSortType ~= self.FILLTYPE_SORT_INDEX then
			local sortKey = self.fillTypeSortTypes[self.FILLTYPE_SORT_NAME]
			sourceTable = sortTable(ucMod.fillTypesByIndex, sortKey)

			if self.fillTypeSortType == self.FILLTYPE_SORT_GROUP then
				local newSourceTable = {}
				for _, group in ipairs(ucMod.fillTypeGroupsByPriority) do
					for _, fillType in ipairs(sourceTable) do
						if fillType.group == group.index then
							table.insert(newSourceTable, fillType)
						end
					end
				end
				sourceTable = newSourceTable
			end
		end

		template = self.fillTypesListItemTemplate

		-- This needs to happen every time the fillType list gets updated to support adding and removing groups
		list.groupValues = {}
		list.groupMapping = {}
		for _, fillTypeGroup in ipairs(ucMod.fillTypeGroupsByPriority) do
			table.insert(list.groupValues, {value = fillTypeGroup.index, text = fillTypeGroup.name})
			list.groupMapping[fillTypeGroup.index] = getTableLength(list.groupValues)
		end
	elseif list == self.unitsList then
		list.unitMapping = {}
		list.highlight = self.unitsBoxHighlight

		sourceTable = ucMod.unitsByIndex
		template = self.unitsListItemTemplate
	elseif list == self.globalList then
		list.unitTypeMapping = {}
		list.highlight = self.globalBoxHighlight

		sourceTable = ucMod.unitTypesByIndex
		template = self.globalListItemTemplate
	elseif list == self.groupsList then
		list.groupMapping = {}
		list.highlight = self.groupsBoxHighlight

		sourceTable = ucMod.fillTypeGroupsByPriority
		template = self.groupsListItemTemplate
	else
		raiseError()
		return
	end

	local idx = 1
	for _, target in ipairs(sourceTable) do
		if (list == self.fillTypesList and target.showOnMenu)
		or (list == self.unitsList and target.unitType ~= ucMod.UNIT_TYPE_PIECE)
		or (list == self.globalList and target.index ~= ucMod.UNIT_TYPE_PIECE)
		or (list == self.groupsList)
		then
			local listItem = template:clone(list)
			listItem.listIndex = idx

			for i, element in ipairs(listItem.elements[1].elements) do
				if idx % 2 == 0 then
					element:applyProfile("uc_listItemBgEven")
				end

				element.listItem = listItem

				if list == self.fillTypesList then
					local fillType = target
					listItem.fillTypeIndex = fillType.index
					list.fillTypeMapping[fillType.index] = listItem.listIndex
					if i == 1 then -- FillType Name and Icon
						local itemIcon = element.elements[1]
						local itemName = element.elements[2]
						local fillTypeIcon = FillUtil.fillTypeIndexToDesc[fillType.index].hudOverlayFilenameSmall
						if fillTypeIcon and fillTypeIcon ~= "" then
							itemIcon:setImageFilename(fillTypeIcon)
						else
							itemIcon:setImageFilename("dataS2/menu/hud/fillTypes/hud_fill_unknown_sml.png")
						end
						itemName:setText(fillType.name)
					elseif i == 2 then -- FillType Group
						local fillTypeGroupIndex = list.groupMapping[fillType.group]
						if xAssert(fillTypeGroupIndex) then
							element.valueIndex = fillTypeGroupIndex
						end
						element.textElement:setText(list.groupValues[element.valueIndex].text)
					elseif i == 3 then -- FillType Mass
						local bulkDensity = Utils.getNoNil(fillType.bulkDensity, ucMod:massPerLiterToBulkDensity(fillType.massPerLiter))
						local fillTypeDensityIndex = list.densityMapping[bulkDensity]
						if xAssert(fillTypeDensityIndex) then
							element.valueIndex = fillTypeDensityIndex
						end
						self:setFillTypeMassText(element)
					end
				elseif list == self.unitsList then
					local unit = target
					listItem.unitIndex = unit.index
					list.unitMapping[unit.index] = listItem.listIndex
					if i == 1 then -- Unit Name and Icon
						local itemName = element.elements[2]
						itemName:setText(unit.name)
					elseif i == 2 then -- Unit Precision
						local precisionIndex = list.precisionMapping[unit.precision]
						if xAssert(precisionIndex) then
							element.valueIndex = precisionIndex
						end
						element.textElement:setText(list.precisionValues[element.valueIndex].text)
					end
				elseif list == self.globalList then
					local unitType = target
					listItem.unitTypeIndex = unitType.index
					list.unitTypeMapping[unitType.index] = listItem.listIndex
					if i == 1 then -- UnitType name and icon
						local itemName = element.elements[2]
						itemName:setText(unitType.name)
					elseif i == 2 then -- UnitType default unit
						element.unitValues = {}
						element.unitMapping = {}
						for _, unit in ipairs(ucMod.unitsByIndex) do
							if unit.unitType == unitType.index then
								table.insert(element.unitValues, {value = unit.index, text = unit.name})
								element.unitMapping[unit.index] = getTableLength(element.unitValues)
							end
						end
						local unitIndex = element.unitMapping[unitType.defaultUnit]
						if xAssert(unitIndex) then
							element.valueIndex = unitIndex
						end
						element.textElement:setText(element.unitValues[element.valueIndex].text)
					end
				elseif list == self.groupsList then
					local fillTypeGroup = target
					listItem.groupIndex = fillTypeGroup.index
					list.groupMapping[fillTypeGroup.index] = listItem.listIndex
					if i == 1 then -- Group Name and Icon
						local itemName = element.elements[2]
						itemName:setText(fillTypeGroup.name)
					elseif i >= 2 and i <= 5 then
						element.unitValues = {}
						element.unitMapping = {}
						if i == 2 then
							element.unitSet = ucMod.UNIT_SET_VEHICLE
						elseif i == 3 then
							element.unitSet = ucMod.UNIT_SET_STORAGE
						elseif i == 4 then
							element.unitSet = ucMod.UNIT_SET_ANIMAL
						elseif i == 5 then
							element.unitSet = ucMod.UNIT_SET_PRICE
						end
						for _, unit in pairs(ucMod.unitsByIndex) do
							if fillTypeGroup.unitTypes[unit.unitType] == true then
								table.insert(element.unitValues, {value = unit.index, text = unit.name})
								element.unitMapping[unit.index] = getTableLength(element.unitValues)
							end
						end
						local unitIndex = element.unitMapping[fillTypeGroup.units[element.unitSet]]
						if xAssert(unitIndex) then
							element.valueIndex = unitIndex
						end
						element.textElement:setText(element.unitValues[element.valueIndex].text)
					end
				end
			end
			idx = idx + 1
		end
	end
	self:selectListItem(list, selectedItem, true, sliderValue)
end

function UCMenu:updateList(list, dt)
-- This function is for displaying fillType information as well as highlighting the selected (focused) list
	-- [list]: List to update
	-- [dt]:   Delta time (time since last update)

	if not getIsTable(list) then
		raiseError()
		return
	end

	local listItem = list:getSelectedElement()

	if list.highlight then
		if FocusManager:hasFocus(list)
		or (list == self.fillTypesList
			and FocusManager:hasFocus(self.resetFillTypeMassButton))
		or (list == self.groupsList
			and (FocusManager:hasFocus(self.groupPriorityUpButton)
			or FocusManager:hasFocus(self.groupPriorityDownButton)
			or FocusManager:hasFocus(self.addGroupButton)
			or FocusManager:hasFocus(self.removeGroupButton)
			or FocusManager:hasFocus(self.addGroupInput)))
		then
			if not list.highlight:getIsVisible() then
				list.highlight:setVisible(true)
			end
		else
			if list.highlight:getIsVisible() then
				list.highlight:setVisible(false)
			end
		end
	end

	if listItem then
		if listItem.state ~= GuiOverlay.STATE_SELECTED then
			listItem:setOverlayState(GuiOverlay.STATE_SELECTED)
		end

		if list == self.fillTypesList then
			local infoBox = self.fillTypesInfoBox

			local titleElement = infoBox.elements[1]
			local massElement = infoBox.elements[2]
			local yieldElement = infoBox.elements[3]

			local fillType = ucMod:getFillType(listItem.fillTypeIndex)
			local fillTypeDensity = ucMod:massPerLiterToBulkDensity(fillType.massPerLiter)
			local fillTypeMassText = getText("UC_menu_fillTypeMass", fillTypeDensity, "kg/m3")

			local fruitTypeYieldText = ""

			local fruitType = FruitUtil.fillTypeToFruitType[fillType.index]
			if fruitType then
				fruitType = FruitUtil.fruitIndexToDesc[fruitType]
				local yield = fruitType.literPerSqm * 10000 -- Convert to literPerHectare

				local areaUnit = ucMod:getDefaultUnit(ucMod.UNIT_TYPE_AREA)
				local fillTypeUnit = ucMod:getFillUnit(fillType.index, ucMod.UNIT_SET_PRICE)

				yield = yield / areaUnit.conversionFactor
				yield = ucMod:convertValue(yield, fillTypeUnit.index, fillType.index)
				yield = numToStr(yield, fillTypeUnit.precision, "ceil", true, false)

				fruitTypeYieldText = getText("UC_menu_fruitTypeYield", yield, fillTypeUnit.nameI18NShort, areaUnit.nameI18NShort)
			end

			titleElement:setText(fillType.nameI18N)
			massElement:setText(fillTypeMassText)
			yieldElement:setText(fruitTypeYieldText)

			if not infoBox:getIsVisible() then
				infoBox:setVisible(true)
			end
		elseif list == self.groupsList then
			local fillTypeGroup = ucMod:getFillTypeGroup(listItem.groupIndex)
			local newGroupText = self.addGroupInput.text

			-- Enable/disable add group button
			if not newGroupText or newGroupText == "" or ucMod.fillTypeGroupsByName[newGroupText] then
				if not self.addGroupButton.disabled then
					self.addGroupButton:setDisabled(true)
				end
			else
				if self.addGroupButton.disabled then
					self.addGroupButton:setDisabled(false)
				end
			end

			-- Enable/disable remove group button
			if fillTypeGroup.isDefault then
				if not self.removeGroupButton.disabled then
					self.removeGroupButton:setDisabled(true)
				end
			else
				if self.removeGroupButton.disabled then
					self.removeGroupButton:setDisabled(false)
				end
			end
		end
	else
		if list == self.fillTypesList then
			if self.fillTypesInfoBox:getIsVisible() then
				self.fillTypesInfoBox:setVisible(false)
			end
		elseif list == self.groupsList then
			if not self.removeGroupButton.disabled then
				self.removeGroupButton:setDisabled(true)
			end
		end
	end
end

function UCMenu:selectListItem(list, rowIndex, force, sliderValue)
-- This is a wrapper function for selecting a list item (better slider handling)
	-- [list]:        Target list for selection
	-- [rowIndex]:    Row to select
	-- [force]:       Not sure, but I assume it means force selection on true
	-- [sliderValue]: Value to set the slider to upon completion

	if not getIsTable(list) or not getIsNumber(rowIndex) or (force and not getIsBoolean(force)) or (sliderValue and not getIsNumber(sliderValue)) then
		raiseError()
		return
	end

	list:setSelectedRow(rowIndex, force)

	local slider = list.parent
	if not sliderValue then
		sliderValue = slider.maxValue - (rowIndex - 1)
	end
	sliderValue = Utils.clamp(sliderValue, slider.minValue, slider.maxValue)
	slider:setValue(sliderValue)
end

function UCMenu:getIsListItemVisible(list, rowIndex)
-- This is a wrapper function to determine if a particular list item is visible or not
	-- [list]:     Target list
	-- [rowIndex]: Target row

	if not getIsTable(list) or not getIsNumber(rowIndex) then
		raiseError()
		return
	end

	return rowIndex >= list.firstVisibleItem and rowIndex <= (list.firstVisibleItem + (list.visibleItems - 1))
end

-- [FILLTYPES] ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onCreateFillTypesList(element)
	local list = element

	list.densityValues = {}
	list.densityMapping = {}
	for i = ucMod.FILLTYPE_BULKDENSITY_MIN, ucMod.FILLTYPE_BULKDENSITY_MAX, ucMod.FILLTYPE_BULKDENSITY_STEP do
		table.insert(list.densityValues, {value = i, text = tostring(i).." kg/m3"})
		list.densityMapping[i] = getTableLength(list.densityValues)
	end
end

function UCMenu:setFillTypeMassText(element)
-- This function allows formatting the fillType display mass text based on its value
	-- [element]: Element containing fillType data

	local list = self.fillTypesList
	local listItem = element.listItem

	local fillType = ucMod:getFillType(listItem.fillTypeIndex)
	local defaultDensity = ucMod:massPerLiterToBulkDensity(fillType.massPerLiter)
	local currentDensity = list.densityValues[element.valueIndex]

	if currentDensity.value ~= defaultDensity then
		if currentDensity.value > defaultDensity then
			element.textElement:applyProfile("uc_listItemTextGreater")
		else
			element.textElement:applyProfile("uc_listItemTextLesser")
		end
	else
		element.textElement:applyProfile("uc_listItemTextValue2")
	end

	element.textElement:setText(currentDensity.text)
end

function UCMenu:onClickSortFillTypesByName(element)
	local list = self.fillTypesList
	local listItem = list:getSelectedElement()

	if self:hasDoubleClick(element) then
		if self.fillTypeSortType ~= self.FILLTYPE_SORT_NAME then
			self.fillTypeSortType = self.FILLTYPE_SORT_NAME
		else
			self.fillTypeSortType = self.FILLTYPE_SORT_INDEX
		end
		self:initializeList(list, 1, true)
	end
end

function UCMenu:onClickSortFillTypesByGroup(element)
	local list = self.fillTypesList
	local listItem = list:getSelectedElement()

	if self:hasDoubleClick(element) then
		if self.fillTypeSortType ~= self.FILLTYPE_SORT_GROUP then
			self.fillTypeSortType = self.FILLTYPE_SORT_GROUP
		else
			self.fillTypeSortType = self.FILLTYPE_SORT_INDEX
		end
		self:initializeList(list, 1, true)
	end
end

function UCMenu:onClickChangeFillTypeGroup(element)
	local list = self.fillTypesList
	local parent = element.parent
	local listItem = parent.listItem

	local value = parent.valueIndex
	value = value + element.valueDirection
	if value > getTableLength(list.groupValues) then value = 1 end
	if value < 1 then value = getTableLength(list.groupValues) end
	parent.valueIndex = value

	local fillType = ucMod:getFillType(listItem.fillTypeIndex)
	local currentGroup = list.groupValues[parent.valueIndex]

	fillType.group = currentGroup.value
	parent.textElement:setText(currentGroup.text)
end

function UCMenu:onClickChangeFillTypeMass(element)
	local list = self.fillTypesList
	local parent = element.parent
	local listItem = parent.listItem

	local valueDirection = element.valueDirection
	if InputBinding.areKeysPressed(specialFunctionKey) then
		valueDirection = valueDirection * 20
	end

	local value = parent.valueIndex
	value = value + valueDirection
	if value > getTableLength(list.densityValues) then value = 1 end
	if value < 1 then value = getTableLength(list.densityValues) end
	parent.valueIndex = value

	local fillType = ucMod:getFillType(listItem.fillTypeIndex)
	local defaultDensity = ucMod:massPerLiterToBulkDensity(fillType.massPerLiter)
	local currentDensity = list.densityValues[parent.valueIndex]

	if currentDensity == defaultDensity then
		fillType.bulkDensity = nil
	else
		fillType.bulkDensity = currentDensity.value
	end
	self:setFillTypeMassText(parent)
end

function UCMenu:onClickResetFillTypeMass(element)
	local list = self.fillTypesList
	local listItem = list:getSelectedElement()

	if InputBinding.areKeysPressed(specialFunctionKey) then
		local function resetAll(yes)
			if yes then
				for _, fillType in pairs(ucMod.fillTypesByIndex) do
					fillType.bulkDensity = nil
				end
				self:initializeList(list, 1, true)
			end
		end
		g_gui:showYesNoDialog({text = getText("UC_dialog_resetAllMassValuesText"), callback = resetAll})
	else
		local fillType = ucMod:getFillType(listItem.fillTypeIndex)
		fillType.bulkDensity = nil
		self:initializeList(list, listItem.listIndex)
	end
end

-- [UNITS] --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onCreateUnitsList(element)
	local list = element

	list.precisionValues = {}
	list.precisionMapping = {}
	for i = ucMod.UNIT_PRECISION_MIN, ucMod.UNIT_PRECISION_MAX, ucMod.UNIT_PRECISION_STEP do
		table.insert(list.precisionValues, {value = i, text = tostring(i)})
		list.precisionMapping[i] = getTableLength(list.precisionValues)
	end
end

function UCMenu:onClickChangeUnitPrecision(element)
	local list = self.unitsList
	local parent = element.parent
	local listItem = parent.listItem

	local value = parent.valueIndex
	value = value + element.valueDirection
	if value > getTableLength(list.precisionValues) then value = 1 end
	if value < 1 then value = getTableLength(list.precisionValues) end
	parent.valueIndex = value

	local unit = ucMod:getUnit(listItem.unitIndex)
	local currentPrecision = list.precisionValues[parent.valueIndex]

	unit.precision = currentPrecision.value
	parent.textElement:setText(currentPrecision.text)
end

-- [GLOBAL] ------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onClickChangeGlobalUnit(element)
	local parent = element.parent
	local listItem = parent.listItem

	local value = parent.valueIndex
	value = value + element.valueDirection
	if value > getTableLength(parent.unitValues) then value = 1 end
	if value < 1 then value = getTableLength(parent.unitValues) end
	parent.valueIndex = value

	local unitType = ucMod:getUnitType(listItem.unitTypeIndex)
	local currentUnit = parent.unitValues[parent.valueIndex]

	unitType.defaultUnit = currentUnit.value
	parent.textElement:setText(currentUnit.text)
end

-- [SETTINGS] -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:initializeSettingsList()
-- This function simply initializes the settings list

	local list = self.settingsList
	list.highlight = self.settingsBoxHighlight

	for listIdx, listItem in ipairs(list.listItems) do
		listItem.listIndex = listIdx

		local settingsElement = listItem.elements[1].elements[2]
		settingsElement.listItem = listItem

		if listIdx == 1 then -- Yield display setting
			settingsElement.valueIndex = ucMod.yieldDisplay
			settingsElement.textElement:setText(ucMod.yieldDisplayTypes[settingsElement.valueIndex])
		end
	end
end

function UCMenu:onClickChangeYieldDisplay(element)
	local list = self.settingsList
	local parent = element.parent
	local listItem = parent.listItem

	local value = parent.valueIndex
	value = value + element.valueDirection
	if value > getTableLength(ucMod.yieldDisplayTypes) then value = 1 end
	if value < 1 then value = getTableLength(ucMod.yieldDisplayTypes) end
	parent.valueIndex = value

	ucMod.yieldDisplay = parent.valueIndex
	parent.textElement:setText(ucMod.yieldDisplayTypes[parent.valueIndex])
end

-- [FILLTYPE GROUPS] ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

function UCMenu:onClickChangeGroupUnit(element)
	local list = self.groupsList
	local parent = element.parent
	local listItem = parent.listItem

	local fillTypeGroup = ucMod:getFillTypeGroup(listItem.groupIndex)
	if fillTypeGroup.lockUnits then
		return
	end

	local value = parent.valueIndex
	value = value + element.valueDirection
	if value > getTableLength(parent.unitValues) then value = 1 end
	if value < 1 then value = getTableLength(parent.unitValues) end
	parent.valueIndex = value

	local currentUnit = parent.unitValues[parent.valueIndex]

	if InputBinding.areKeysPressed(specialFunctionKey) then
		for unitSet, _ in pairs(fillTypeGroup.units) do
			fillTypeGroup.units[unitSet] = currentUnit.value
		end
		self:initializeList(list, listItem.listIndex)
	else
		fillTypeGroup.units[parent.unitSet] = currentUnit.value
		parent.textElement:setText(currentUnit.text)
	end
end

function UCMenu:onClickChangeGroupPriority(element)
	local list = self.groupsList
	local listItem = list:getSelectedElement()

	local fillTypeGroup = ucMod:getFillTypeGroup(listItem.groupIndex)
	local oldValue = fillTypeGroup.priority
	local newValue = oldValue

	if InputBinding.areKeysPressed(specialFunctionKey) then
		if element.valueDirection == 1 then
			newValue = getTableLength(list.listItems)
		else
			newValue = 1
		end
	else
		newValue = newValue + element.valueDirection
		if newValue > getTableLength(list.listItems) then newValue = getTableLength(list.listItems) end
		if newValue < 1 then newValue = 1 end
	end

	local oldGroup = table.remove(ucMod.fillTypeGroupsByPriority, oldValue)
	table.insert(ucMod.fillTypeGroupsByPriority, newValue, oldGroup)

	for idx, fillTypeGroup in ipairs(ucMod.fillTypeGroupsByPriority) do
		fillTypeGroup.priority = idx
	end

	self:initializeList(list, newValue)

	if self.fillTypeSortType == self.FILLTYPE_SORT_GROUP then
		self:initializeList(self.fillTypesList, 1, true)
	end
end

function UCMenu:onClickAddRemoveGroup(element)
	local list = self.groupsList
	local listItem = list:getSelectedElement()
	local fillTypeGroup = ucMod:getFillTypeGroup(listItem.groupIndex)

	if element.valueDirection > 0 then -- Add
		local newGroupText = self.addGroupInput.text
		if newGroupText and newGroupText ~= "" and not ucMod.fillTypeGroupsByName[newGroupText] then
			ucMod:addFillTypeGroup(newGroupText)

			self.addGroupInput:setText("")

			local selectedRow = getTableLength(list.listItems) + 1

			self:initializeList(self.groupsList, selectedRow, true)
			self:initializeList(self.fillTypesList)
		end
	else -- Remove
		local function removeGroup(yes)
			if yes then
				ucMod:removeFillTypeGroup(fillTypeGroup.name)

				self:initializeList(self.groupsList)
				self:initializeList(self.fillTypesList)
			end
		end
		g_gui:showYesNoDialog({text = getText("UC_dialog_removeGroupText", fillTypeGroup.name), callback = removeGroup})
	end
end
